
import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashMap;
import java.util.Map;
/**
 * 
 * @author Giovanni Cuccu
 * @version $Revision$ - $Date$
 */
public class OracleUtilities  {
    private final static String SESSION_STAT_QUERY="Select sn.name,st.value from v$mystat st,v$statname sn where st.statistic#=sn.statistic#";
    private final static String STAT_QUERY_FILTER=" where name like ?";
    private final static String STAT_QUERY_ORDER=" order by name";
    private OracleUtilities() {
    }
    
    public static void flushSharedPool(String systemUsername,String systemPassword) throws SQLException {
        DriverManager.registerDriver(new oracle.jdbc.driver.OracleDriver());
        Connection conn = DriverManager.getConnection(
            "jdbc:oracle:thin:@localhost:1521:orcl",systemUsername,systemPassword);
        try {
            Statement st=conn.createStatement();
            try {
                st.execute("alter system flush shared_pool");
            } finally {
                st.close();
            }
        } finally {
            conn.close();
        }
        
    }
    
    public static void takeStatsPackSnapshot(String statsPackUsername,String statsPackPassword) throws SQLException {
        DriverManager.registerDriver(new oracle.jdbc.driver.OracleDriver());
        Connection conn = DriverManager.getConnection(
            "jdbc:oracle:thin:@localhost:1521:orcl",statsPackUsername,statsPackPassword);
        try {
            Statement st=conn.createStatement();
            CallableStatement takeSnapshot=conn.prepareCall("{call statspack.snap}");
            try {
                takeSnapshot.execute();
            } finally {
                st.close();
            }
        } finally {
            conn.close();
        }
        
    }    
    
    public static void enableTrace(Connection connection, String identifier) throws SQLException{
        java.sql.Statement st=connection.createStatement();
        try {
            st.execute("Alter session set timed_statistics=true");
        } finally {
            st.close();
        }
        st=connection.createStatement();
        try {
            st.execute("Alter session set tracefile_identifier='" + identifier + "'");
        } finally {
            st.close();
        }
        st=connection.createStatement();
        try {
            st.execute("Alter session set events '10046 trace name context forever, level 12'");
        } finally {
            st.close();
        }
        insertTimeStampInTraceFile(connection);
    }
    
    public static void insertTimeStampInTraceFile(Connection connection) throws SQLException{
        CallableStatement procedure=connection.prepareCall("{call sys.dbms_system.ksdddt}");
        try {
            procedure.execute();
        } finally {
            procedure.close();
        }
    }
    
    public static Map getSessionStatistics(Connection connection, String statNameFilter) throws SQLException{
        Map map=new HashMap();
        if (statNameFilter!=null && !statNameFilter.equals("")) {
            PreparedStatement ps=connection.prepareStatement(SESSION_STAT_QUERY + STAT_QUERY_FILTER+ STAT_QUERY_ORDER);
            try {
                ps.setString(1,"%"+statNameFilter+"%");
                ResultSet rs=ps.executeQuery();
                try {
                    while (rs.next()) {
                        map.put(rs.getString(1),rs.getString(2));
                    }
                } finally {
                    rs.close();
                }
            } finally {
                ps.close();
            }
            return map;
        } else {
            Statement st=connection.createStatement();
            try {
                ResultSet rs=st.executeQuery(SESSION_STAT_QUERY + STAT_QUERY_ORDER);
                try {
                    while (rs.next()) {
                        map.put(rs.getString(1),rs.getString(2));
                    }
                
                } finally {
                    rs.close();
                }
            } finally {
                st.close();
            } 
            return map;
        }
    }
}