
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.Statement;
import java.text.SimpleDateFormat;
import oracle.jdbc.OracleConnection;
import org.apache.log4j.Logger;
/**
 * 
 * @author Giovanni Cuccu
 * @version $Revision$ - $Date$
 */
public class TestInsert  {
    private static final Logger logger=Logger.getLogger(TestRealInsert2.class.getName());
    public TestInsert() {
    }

    public static void main(String[] args) throws Exception{
        TestInsert testInsert = new TestInsert();
        int repetitions=1;
        testInsert.testBindVsNoBindV3("system","giovanni",repetitions,500);
    
    }
    
    public void testBindVsNoBind(String systemUsername,String systemPassword, 
            int testsNo, int insertNo) throws Exception {
        long bindElapsed[]=new long[testsNo];
        long noBindElapsed[]=new long[testsNo];
        double bindAverage=0;
        double noBindAverage=0;
        for (int i=0;i<testsNo;i++) {
            OracleUtilities.flushSharedPool(systemUsername,systemPassword);
            noBindAverage+=testInsertOReillyWithNoBinds(insertNo);    
        }
        noBindAverage=noBindAverage/testsNo;
        for (int i=0;i<testsNo;i++) {
            OracleUtilities.flushSharedPool(systemUsername,systemPassword);
            bindAverage+=testInsertOReillyWithBinds(insertNo);    
        }
        bindAverage=bindAverage/testsNo;
        System.out.println("test results for " +  insertNo+ " insert");
        System.out.println("NoBindAverage " +  noBindAverage);
        System.out.println("bindAverage " +  bindAverage);
    }
    
    public void testBindVsNoBindV2(String systemUsername,String systemPassword, 
            int testsNo, int insertNo) throws Exception {
        long bindElapsed[]=new long[testsNo];
        long noBindElapsed[]=new long[testsNo];
        double bindAverage=0;
        double noBindAverage=0;
        for (int i=0;i<testsNo;i++) {
            OracleUtilities.flushSharedPool(systemUsername,systemPassword);
            noBindAverage+=testInsertOReillyWithNoBinds(insertNo);    
        }
        noBindAverage=noBindAverage/testsNo;
        for (int i=0;i<testsNo;i++) {
            OracleUtilities.flushSharedPool(systemUsername,systemPassword);
            bindAverage+=testInsertOReillyWithBindsV2(insertNo); 
            System.out.println("BindAverage int step" +  bindAverage);
        }
        bindAverage=bindAverage/testsNo;
        System.out.println("test results for " +  insertNo+ " insert");
        System.out.println("NoBindAverage " +  noBindAverage);
        System.out.println("BindAverage " +  bindAverage);
    }
    
    public void testBindVsNoBindV3(String systemUsername,String systemPassword, 
            int testsNo, int insertNo) throws Exception {
        long bindElapsed[]=new long[testsNo];
        long noBindElapsed[]=new long[testsNo];
        double bindAverage=0;
        double noBindAverage=0;
        for (int i=0;i<testsNo;i++) {
            OracleUtilities.flushSharedPool(systemUsername,systemPassword);
            noBindAverage+=testInsertOReillyWithNoBindsAndTrace(insertNo);    
        }
        noBindAverage=noBindAverage/testsNo;
        for (int i=0;i<testsNo;i++) {
            OracleUtilities.flushSharedPool(systemUsername,systemPassword);
            bindAverage+=testInsertOReillyWithBindsV2AndTrace(insertNo); 
            System.out.println("BindAverage int step" +  bindAverage);
        }
        bindAverage=bindAverage/testsNo;
        System.out.println("test results for " +  insertNo+ " insert");
        System.out.println("NoBindAverage " +  noBindAverage);
        System.out.println("BindAverage " +  bindAverage);
    }    
    
    
    private long testInsertOReillyWithNoBinds(int repetitions) throws Exception {
        int i;
        int n;
        long startTime; 
        long stopTime;
        
        n = repetitions;
        String[] codes  = new String[n];
        String[] descrs = new String[n];
        for (i=0;i<n;i++) {
            byte[] bytes = new byte[30];
            for (int x=0;x<30;x++) {
                bytes[x] = (byte)((java.lang.Math.random() * 74) + 48);
            }
            codes[i] = new String(bytes);
            bytes = new byte[80];
            for (int x=0;x<80;x++) {
                bytes[x] = (byte)((java.lang.Math.random() * 74) + 48);
            }
            descrs[i] = new String(bytes);
        }
        DriverManager.registerDriver(new oracle.jdbc.driver.OracleDriver());
        Connection conn = DriverManager.getConnection(
            "jdbc:oracle:thin:@localhost:1521:orcl","test","test");
        Statement stmt = conn.createStatement();
        SimpleDateFormat sdf = new SimpleDateFormat( "yyyyMMddHHmmss" );
        conn.setAutoCommit(false);
        
        startTime = System.currentTimeMillis();
        for (i=0;i<n;i++) {
            stmt.executeUpdate("insert into testxxxperf ( id, code, descr, insert_user, insert_date ) " +
                "values ( " + Integer.toString( i ) + ", '" + codes[i] + "', '" + descrs[i] + "', " +
                "USER, to_date('" + sdf.format(new java.util.Date(System.currentTimeMillis())) + "', 'YYYYMMDDHH24MISS'))");
        }
        stmt.close();
        conn.commit(); 
        stopTime = System.currentTimeMillis();
        Statement dstmt = conn.createStatement();
        dstmt.executeUpdate("delete testxxxperf");
        conn.commit();
        dstmt.close();
        stmt.close();
        conn.close();
        return (stopTime-startTime);
    }
    
    private long testInsertOReillyWithNoBindsAndTrace(int repetitions) throws Exception {
        int i;
        int n;
        long startTime; 
        long stopTime;
        
        n = repetitions;
        String[] codes  = new String[n];
        String[] descrs = new String[n];
        for (i=0;i<n;i++) {
            byte[] bytes = new byte[30];
            for (int x=0;x<30;x++) {
                bytes[x] = (byte)((java.lang.Math.random() * 74) + 48);
            }
            codes[i] = new String(bytes);
            bytes = new byte[80];
            for (int x=0;x<80;x++) {
                bytes[x] = (byte)((java.lang.Math.random() * 74) + 48);
            }
            descrs[i] = new String(bytes);
        }
        DriverManager.registerDriver(new oracle.jdbc.driver.OracleDriver());
        Connection conn = DriverManager.getConnection(
            "jdbc:oracle:thin:@localhost:1521:orcl","test","test");
        Statement stmt = conn.createStatement();
        SimpleDateFormat sdf = new SimpleDateFormat( "yyyyMMddHHmmss" );
        conn.setAutoCommit(false);
        OracleUtilities.enableTrace(conn,"use_statement");
        startTime = System.currentTimeMillis();
        for (i=0;i<n;i++) {
            stmt.executeUpdate("insert into testxxxperf ( id, code, descr, insert_user, insert_date ) " +
                "values ( " + Integer.toString( i ) + ", '" + codes[i] + "', '" + descrs[i] + "', " +
                "USER, to_date('" + sdf.format(new java.util.Date(System.currentTimeMillis())) + "', 'YYYYMMDDHH24MISS'))");
        }
        stmt.close();
        conn.commit(); 
        stopTime = System.currentTimeMillis();
        Statement dstmt = conn.createStatement();
        dstmt.executeUpdate("delete testxxxperf");
        conn.commit();
        dstmt.close();
        stmt.close();
        conn.close();
        return (stopTime-startTime);
    }    

    private long testInsertOReillyWithBinds(int repetitions) throws Exception {
        int i;
        int n;
        long startTime; 
        long stopTime;
        
        n = repetitions;
        String[] codes  = new String[n];
        String[] descrs = new String[n];
        for (i=0;i<n;i++) {
            byte[] bytes = new byte[30];
            for (int x=0;x<30;x++) {
                bytes[x] = (byte)((java.lang.Math.random() * 74) + 48);
            }
            codes[i] = new String(bytes);
            bytes = new byte[80];
            for (int x=0;x<80;x++) {
                bytes[x] = (byte)((java.lang.Math.random() * 74) + 48);
            }
            descrs[i] = new String(bytes);
        }
        DriverManager.registerDriver(new oracle.jdbc.driver.OracleDriver());
        Connection conn = DriverManager.getConnection(
            "jdbc:oracle:thin:@localhost:1521:orcl","test","test");
        conn.setAutoCommit(false);
        startTime = System.currentTimeMillis();
        PreparedStatement stmt = conn.prepareStatement(
        "insert into testxxxperf ( id, code, descr, insert_user, insert_date ) " +
        "values ( ?, ?, ?, USER, ? )");
        for (i=0;i<n;i++) {
            stmt.setInt(1,i);
            stmt.setString(2,codes[i]);
            stmt.setString(3,descrs[i]);
            stmt.setDate(4,new java.sql.Date(System.currentTimeMillis()));
            stmt.executeUpdate();
        }
        stmt.close();
        conn.commit(); 
        stopTime = System.currentTimeMillis();
        Statement dstmt = conn.createStatement();
        dstmt.executeUpdate("delete testxxxperf");
        conn.commit();
        dstmt.close();
        stmt.close();
        conn.close();
        return (stopTime-startTime);
    }
    
    
    /*private long testInsertOReillyWithBindsV2(int start, int repetitions) throws Exception {
        int i;
        int n;
        long startTime; 
        long stopTime;
        
        n = repetitions;
        String[] codes  = new String[n];
        String[] descrs = new String[n];
        for (i=0;i<n;i++) {
            byte[] bytes = new byte[30];
            for (int x=0;x<30;x++) {
                bytes[x] = (byte)((java.lang.Math.random() * 74) + 48);
            }
            codes[i] = new String(bytes);
            bytes = new byte[80];
            for (int x=0;x<80;x++) {
                bytes[x] = (byte)((java.lang.Math.random() * 74) + 48);
            }
            descrs[i] = new String(bytes);
        }
        DriverManager.registerDriver(new oracle.jdbc.driver.OracleDriver());
        Connection conn = DriverManager.getConnection(
            "jdbc:oracle:thin:@localhost:1521:orcl","test","test");
        conn.setAutoCommit(false);
        PreparedStatement stmt = conn.prepareStatement(
        "insert into testxxxperf ( id, code, descr, insert_user, insert_date ) " +
        "values ( ?, ?, ? , USER, ? )");
        startTime = System.currentTimeMillis();
        for (i=0;i<n;i++) {
            stmt.setInt(1,i);
            stmt.setString(2,codes[i]);
            stmt.setString(3,descrs[i]);
            stmt.setDate(4,new java.sql.Date(System.currentTimeMillis()));
            stmt.executeUpdate();
        }
        stmt.close();
        conn.commit(); 
        stopTime = System.currentTimeMillis();
        Statement dstmt = conn.createStatement();
        dstmt.executeUpdate("delete testxxxperf");
        conn.commit();
        dstmt.close();
        stmt.close();
        conn.close();
        return (stopTime-startTime);
    } */   
    
    private long testInsertOReillyWithBindsV2(int repetitions) throws Exception {
        int i;
        int n;
        long startTime; 
        long stopTime;
        
        n = repetitions;
        String[] codes  = new String[n];
        String[] descrs = new String[n];
        for (i=0;i<n;i++) {
            byte[] bytes = new byte[30];
            for (int x=0;x<30;x++) {
                bytes[x] = (byte)((java.lang.Math.random() * 74) + 48);
            }
            codes[i] = new String(bytes);
            bytes = new byte[80];
            for (int x=0;x<80;x++) {
                bytes[x] = (byte)((java.lang.Math.random() * 74) + 48);
            }
            descrs[i] = new String(bytes);
        }
        DriverManager.registerDriver(new oracle.jdbc.driver.OracleDriver());
        Connection conn = DriverManager.getConnection(
            "jdbc:oracle:thin:@localhost:1521:orcl","test","test");
        //OracleUtilities.enableTrace(conn,"reuse_cursor");
        conn.setAutoCommit(false);
        logger.debug("Statement caching: " + ((OracleConnection)conn).getImplicitCachingEnabled());
        PreparedStatement stmt1 = conn.prepareStatement(
        "select 1 from dual vhere x=?");   
        startTime = System.currentTimeMillis();
        PreparedStatement stmt = conn.prepareStatement(
        "insert into testxxxperf ( id, code, descr, insert_user, insert_date ) " +
        "values ( ?, ? , ? , USER , ? )");
        long startSQL=0;
        long endSQL=0;
        startSQL=System.currentTimeMillis();
        for (i=0;i<n;i++) {
            stmt.setInt(1,i);
            stmt.setString(2,codes[i]);
            stmt.setString(3,descrs[i]);
            stmt.setDate(4,new java.sql.Date(System.currentTimeMillis()));
            stmt.execute();
            stmt.clearParameters();
        }
        endSQL=System.currentTimeMillis();
        stmt.close();
        conn.commit(); 
        stopTime = System.currentTimeMillis();
        logger.debug("Reusing PS SQLTime:" + (endSQL-startSQL));
        Statement dstmt = conn.createStatement();
        dstmt.executeUpdate("delete testxxxperf");
        conn.commit();
        dstmt.close();
        stmt.close();
        conn.close();
        return (stopTime-startTime);
    }
    
    private long testInsertOReillyWithBindsV2AndTrace(int repetitions) throws Exception {
        int i;
        int n;
        long startTime; 
        long stopTime;
        
        n = repetitions;
        String[] codes  = new String[n];
        String[] descrs = new String[n];
        for (i=0;i<n;i++) {
            byte[] bytes = new byte[30];
            for (int x=0;x<30;x++) {
                bytes[x] = (byte)((java.lang.Math.random() * 74) + 48);
            }
            codes[i] = new String(bytes);
            bytes = new byte[80];
            for (int x=0;x<80;x++) {
                bytes[x] = (byte)((java.lang.Math.random() * 74) + 48);
            }
            descrs[i] = new String(bytes);
        }
        DriverManager.registerDriver(new oracle.jdbc.driver.OracleDriver());
        Connection conn = DriverManager.getConnection(
            "jdbc:oracle:thin:@localhost:1521:orcl","test","test");
        //OracleUtilities.enableTrace(conn,"reuse_cursor");
        conn.setAutoCommit(false);
        PreparedStatement stmt1 = conn.prepareStatement(
        "select 1 from dual vhere x=?");   
        OracleUtilities.enableTrace(conn,"use_prepared_statement");
        startTime = System.currentTimeMillis();
        PreparedStatement stmt = conn.prepareStatement(
        "insert into testxxxperf ( id, code, descr, insert_user, insert_date ) " +
        "values ( ?, ? , ? , USER , ? )");
        long startSQL=0;
        long endSQL=0;
        startSQL=System.currentTimeMillis();
        for (i=0;i<n;i++) {
            stmt.setInt(1,i);
            stmt.setString(2,codes[i]);
            stmt.setString(3,descrs[i]);
            stmt.setDate(4,new java.sql.Date(System.currentTimeMillis()));
            stmt.execute();
            stmt.clearParameters();
        }
        endSQL=System.currentTimeMillis();
        stmt.close();
        conn.commit(); 
        stopTime = System.currentTimeMillis();
        logger.debug("Reusing PS SQLTime:" + (endSQL-startSQL));
        Statement dstmt = conn.createStatement();
        dstmt.executeUpdate("delete testxxxperf");
        conn.commit();
        dstmt.close();
        stmt.close();
        conn.close();
        return (stopTime-startTime);
    }    
        
}