
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.Statement;
import java.text.SimpleDateFormat;
/**
 * 
 * @author Giovanni Cuccu
 * @version $Revision$ - $Date$
 */
public class TestMultiUserInsert  {
    private static final int PARAM_ID=1;
    private static final int PARAM_CODE=2;
    private static final int PARAM_DESCR=3;
    private static final int PARAM_DATE=4;

    public TestMultiUserInsert() {
    }

    public static void main(String[] args) throws Exception{
        TestMultiUserInsert testMultiUserInsert = new TestMultiUserInsert();
        String systemUsername="system";
        String systemPassword="giovanni";
        String statsPackUsername="perfstat";
        String statsPackPassword="perfstat";        
        int repetitions=100;
        int threadNo=100;
        OracleUtilities.flushSharedPool(systemUsername,systemPassword);
        OracleUtilities.takeStatsPackSnapshot(statsPackUsername,statsPackPassword);
        testMultiUserInsert.startNoBindsThread(threadNo,repetitions);
        OracleUtilities.takeStatsPackSnapshot(statsPackUsername,statsPackPassword);
        testMultiUserInsert.cleanData();
        OracleUtilities.flushSharedPool(systemUsername,systemPassword);
        OracleUtilities.takeStatsPackSnapshot(statsPackUsername,statsPackPassword);
        testMultiUserInsert.startBindsThread(threadNo,repetitions);
        OracleUtilities.takeStatsPackSnapshot(statsPackUsername,statsPackPassword);
        testMultiUserInsert.cleanData();
    }
    
    private void cleanData() throws Exception{
        Connection connection=createConnection();
        try {
            Statement delete=connection.createStatement();
            try {
                delete.execute("delete testxxxperf");
            } finally { 
                delete.close();
            }
            connection.commit();
        } finally {
            connection.close();
        }
    }
    
    
    private void startBindsThread(int threadNo,int repetitions) throws Exception{
        BindsExecutorThread[] threads=new BindsExecutorThread[threadNo];
        Connection[] connections=new Connection[threadNo];
        try {
            for (int i=0;i<threadNo;i++) {
                connections[i]=createConnection();
                threads[i]=new BindsExecutorThread(connections[i],i*repetitions,repetitions);
                threads[i].start();
            }
            for (int i=0;i<threadNo;i++) {
                threads[i].join();
            }
        } finally {
            for (int i=0;i<threadNo;i++) {
                try {
                connections[i].close();
                } catch (Exception e) {
                    //gestione eccezione
                }
            }
        }
    }
    
    private void startNoBindsThread(int threadNo,int repetitions) throws Exception{
        NoBindsExecutorThread[] threads=new NoBindsExecutorThread[threadNo];
        Connection[] connections=new Connection[threadNo];
        try {
            for (int i=0;i<threadNo;i++) {
                connections[i]=createConnection();
                threads[i]=new NoBindsExecutorThread(connections[i],i*repetitions,repetitions);
                threads[i].start();
            }
            for (int i=0;i<threadNo;i++) {
                threads[i].join();
            }
        } finally {
            for (int i=0;i<threadNo;i++) {
                try {
                connections[i].close();
                } catch (Exception e) {
                    //gestione eccezione
                }
            }
        }
    }    
    
    
    private Connection createConnection() throws Exception {
        DriverManager.registerDriver(new oracle.jdbc.driver.OracleDriver());
        Connection connection=DriverManager.getConnection(
            "jdbc:oracle:thin:@localhost:1521:orcl","test","test");
        connection.setAutoCommit(false);
        return connection;
    }
    

    private void insertWithBinds(Connection connection, int start, int repetitions) throws Exception {
        long startTime = System.currentTimeMillis();
        int id=start-1;
        //logger.debug("Before masterInsert creation");
        PreparedStatement preparedInsert = connection.prepareStatement(
            "insert into testxxxperf ( id, code, descr, insert_user, insert_date ) " +
            "values ( ?, ?, ?, USER, ? )");
        try {
                for (int i=0;i<repetitions;i++) {
                    id++;
                    preparedInsert.setInt(PARAM_ID,id);
                    preparedInsert.setString(PARAM_CODE,"code"+id);
                    preparedInsert.setString(PARAM_DESCR,"descr"+id);
                    preparedInsert.setDate(PARAM_DATE,new java.sql.Date(System.currentTimeMillis()));
                    preparedInsert.executeUpdate();
                }
        } finally {
            preparedInsert.close();
        }
        connection.commit(); 
    }
    
    private void insertWithoutBinds(Connection connection, int start, int repetitions) throws Exception {
        SimpleDateFormat sdf = new SimpleDateFormat( "yyyyMMddHHmmss" );
        int id=start-1;
        for (int i=0;i<repetitions;i++) {
            id++;
            Statement insertStatement = connection.createStatement();
            try {
                insertStatement.executeUpdate("insert into testxxxperf ( id, code, descr, insert_user, insert_date ) " +
                    "values ( " + Integer.toString( id ) + ",'" +  "code" + id + "', '" + "descr" + id + "', " +
                    "USER, to_date('" + sdf.format(new java.util.Date(System.currentTimeMillis())) + "', 'YYYYMMDDHH24MISS'))");
            } finally {
                insertStatement.close();    
            }
        }
        connection.commit();
    }   
    
    private class NoBindsExecutorThread extends Thread {
        private Connection connection;
        private int id;
        private int repetitions;
        
        public NoBindsExecutorThread(Connection connection, int id, int repetitions) {
            this.connection=connection;
            this.id=id;
            this.repetitions=repetitions;
        }
        
        public void run() {
            try {
                insertWithoutBinds(connection,id,repetitions);
            } catch (Exception e) {
                throw new RuntimeException(e);
            }
        }
    }
    
    private class BindsExecutorThread extends Thread {
        private Connection connection;
        private int id;
        private int repetitions;
        
        public BindsExecutorThread(Connection connection, int id, int repetitions) {
            this.connection=connection;
            this.id=id;
            this.repetitions=repetitions;
        }

        public void run() {
            try {
                insertWithBinds(connection,id,repetitions);
            } catch (Exception e) {
                throw new RuntimeException(e);
            }
        }
    }
    
}